<?php

namespace Drupal\monarch_migration_d7\Plugin\MigrationMapper;

use Drupal\monarch_migration\Plugin\MigrationMapper\MigrationMapperBase;

/**
 * The migration mapper plugin base class.
 */
abstract class MigrationMapperD7Base extends MigrationMapperBase {

  /**
   * Get block data.
   */
  protected function getBlockData(string $moddelta = NULL, int $bid = NULL, bool $return_bid_only = FALSE) {
    if (!is_null($moddelta)) {
      $moddelta_parts = explode(':', $moddelta);

      if (count($moddelta_parts) !== 2) {
        return [];
      }
    }
    else {
      $moddelta_parts = [];
    }

    $query = $this->sourceDatabase->select('block', 'b');
    $query->leftJoin('block_custom', 'bc', 'b.bid = bc.bid');
    if ($return_bid_only) {
      $query->fields('b', ['bid']);
    }
    else {
      $query->fields('b')->fields('bc');
    }

    if (!empty($moddelta_parts)) {
      $query->condition('b.module', $moddelta_parts[0]);
      $query->condition('b.delta', $moddelta_parts[1]);
    }

    if (!is_null($bid)) {
      $query->condition('b.bid', $bid);
    }

    if (!empty($moddelta_parts) || !is_null($bid)) {
      $query->range(0, 1);
    }

    $ret = ($return_bid_only ? $query->execute()->fetchCol() : $query->execute()->fetchAll(\PDO::FETCH_ASSOC)) ?: [];

    if (!empty($moddelta_parts) || !is_null($bid)) {
      return current($ret) ?: NULL;
    }

    return $ret;
  }

  /**
   * Lookup migrated block id.
   */
  protected function lookupMigratedBlockId(string $moddelta = NULL, int $bid = NULL) {
    if (is_null($moddelta) && is_null($bid)) {
      return NULL;
    }

    $data = $this->getBlockData($moddelta, $bid, TRUE);
    $ret = $this->migrateLookup->lookup('custom_block', [$data]) ?: [];

    while (is_array($ret)) {
      $ret = reset($ret);
    }

    return $ret === FALSE ? NULL : $ret;
  }

  /**
   * Get the fields for an entity type and bundle.
   */
  public function getBundleFields(string $entity_type_id, string $bundle = NULL, bool $verbose = FALSE) {
    $ret = [];

    $query = $this->sourceDatabase->select('field_config', 'fc');
    $query->innerJoin('field_config_instance', 'fci', 'fci.field_id = fc.id');
    $query->addField('fc', 'field_name');
    $query->addField('fc', 'type');
    $query->condition('entity_type', $entity_type_id);

    if (!is_null($bundle)) {
      $query->condition('bundle', $bundle);
    }

    if ($verbose) {
      $query->addField('fc', 'module');
      $query->addField('fc', 'data', 'storage_data');
      $query->addField('fc', 'cardinality');
      $query->addField('fc', 'translatable');
      $query->addField('fci', 'entity_type');
      $query->addField('fci', 'bundle');
      $query->addField('fc', 'data', 'instance_data');
    }

    $fields = $query->execute()->fetchAll();

    foreach ($fields as &$field) {
      if ($verbose) {
        if (!empty($field->data ?? NULL)) {
          $field->data = unserialize($field->data);
        }

        if (!empty($field->instance_data ?? NULL)) {
          $field->instance_data = unserialize($field->instance_data);
        }

        if (!empty($field->storage_data ?? NULL)) {
          $field->storage_data = unserialize($field->storage_data);
        }

        $ret[$field->field_name] = $field;
      }
      else {
        $ret[$field->field_name] = $field->type;
      }
    }

    return $ret;
  }

  /**
   * Get the field data for a field on an "entity".
   */
  public function getFieldData(string $entity_type_id, string $field_name, ?string $entity_id, ?string $revision_id, ?string $bundle, string $language = 'und') {
    if (!is_null($revision_id)) {
      $query = $this->sourceDatabase->select('field_revision_' . $field_name, 'fd');
      $query->condition('revision_id', (int) $revision_id);
    }
    else {
      $query = $this->sourceDatabase->select('field_data_' . $field_name, 'fd');
    }

    $query->fields('fd');
    $query->condition('entity_type', $entity_type_id);
    $query->condition('deleted', 0);

    if (!is_null($entity_id)) {
      $query->condition('entity_id', (int) $entity_id);
    }

    if (!is_null($bundle)) {
      $query->condition('bundle', $bundle);
    }

    $query->condition('language', $language);
    $query->orderBy('delta', 'ASC');

    $ret = [];

    $rows = $query->execute()->fetchAll();

    foreach ($rows as $row) {
      $delta = $row->delta;

      foreach ($row as $column => $value) {
        if (substr($column, 0, strlen($field_name) + 1) === $field_name . '_') {
          $ret[$delta][substr($column, strlen($field_name) + 1)] = $value;
        }
      }
    }

    return $ret;
  }

  /**
   * Get the all field data for fields on an "entity".
   */
  public function getAllFieldData(string $entity_type_id, string $entity_id, ?string $revision_id, ?string $bundle, string $language = 'und') {
    $ret = [];
    $fields = static::getBundleFields($entity_type_id, $bundle);

    foreach ($fields as $field_name => $field_type) {
      $ret[$field_name] = static::getFieldData($entity_type_id, $field_name, $entity_id, $revision_id, $bundle);
    }

    return $ret;
  }

  /**
   * Get bundle map for all fields.
   */
  public function getSourceConfiguredFieldMap(bool $verbose = FALSE) {
    $ret = [];

    $res = $this->sourceDatabase->select('field_config_instance', 'fci')
      ->fields('fci', ['entity_type', 'bundle'])
      ->distinct()
      ->execute();

    foreach ($res as $row) {
      $ret[$row->entity_type][$row->bundle] = static::getBundleFields($row->entity_type, $row->bundle, $verbose);
    }

    return $ret;
  }

  /**
   * {@inheritdoc}
   */
  public function sourceModuleEnabled(string $module_name) : bool {
    $query = $this->sourceDatabase->select('system', 's');
    $query->condition('name', $module_name);
    $query->condition('status', '1');
    return (int) $query->countQuery()->execute()->fetchField();
  }

}
